#Imports
import numpy as np
import cv2

def plot_boxes(pred_class, pred_boxes, image, path, image_name, color = (0,0,1.0)):
    # converting image numpy array to float for drawing bboxes
    image_f = image.astype('float32')

    # converting from RGB to BGR for using opencv
    image_f = cv2.cvtColor(image_f, cv2.COLOR_RGB2BGR)

    # drawing the top 5 predicted bounding boxes
    for item_number, bbox in enumerate(pred_boxes):
        if(item_number >= 5):
            break
        image_f = cv2.rectangle(image_f, bbox[0], bbox[1], color, 2)
        cv2.putText(image_f, pred_class[item_number], (bbox[0][0], max(bbox[0][1],20)), cv2.FONT_HERSHEY_SIMPLEX, 1, color, 2)
    
    # saving image in outputs folder
    cv2.imwrite(str(path + '\\' + image_name + '.jpg'), 255*image_f)

    # converting from BGR to RGB
    return cv2.cvtColor(image_f, cv2.COLOR_BGR2RGB)
  
  # The function should plot the predicted boxes on the images and save them.
  # Tip: keep the dimensions of the output image less than 800 to avoid RAM crashes.


#Imports
import numpy as np
from skimage.transform import resize
import math

class RescaleImage(object):
    '''
        Rescales the image to a given size.
    '''

    def __init__(self, output_size):
        '''
            Arguments:
            output_size (tuple or int): Desired output size. If tuple, output is
            matched to output_size. If int, smaller of image edges is matched
            to output_size keeping aspect ratio the same.
        '''
        self.output_size = output_size

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)

            Note: You do not need to resize the bounding boxes. ONLY RESIZE THE IMAGE.
        '''
        image_resized = np.array([])
        if(isinstance(self.output_size, tuple)):
            image_resized = resize(image, self.output_size)
        elif(isinstance(self.output_size, int)):
            h, w, _ = image.shape
            if(w < h):
                image_resized = resize(image, (math.floor(h*self.output_size/w), self.output_size))
            else:
                image_resized = resize(image, (self.output_size, math.floor(w*self.output_size/h)))    
        return image_resized



#Imports
import numpy as np

class CropImage(object):
    '''
        Performs either random cropping or center cropping.
    '''

    def __init__(self, shape, crop_type='center'):
        '''
            Arguments:
            shape: output shape of the crop (h, w)
            crop_type: center crop or random crop. Default: center
        '''
        self.shape = shape
        self.crop_type = crop_type

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        h, w, _ = image.shape
        crop_h, crop_w = self.shape
        image_cropped = np.array([])
        start_h = None
        start_w = None

        if(self.crop_type == 'center'):
            start_h = h//2-(crop_h//2)
            start_w = w//2-(crop_w//2)    
            image_cropped = image[start_h : start_h + crop_h, start_w : start_w + crop_w]

        elif(self.crop_type == 'random'):
            start_h = np.random.randint(0, h - crop_h + 1)
            start_w = np.random.randint(0, w - crop_w + 1)
            image_cropped = image[start_h : start_h + crop_h, start_w : start_w + crop_w]
        
        return image_cropped



#Imports
import numpy as np
import json
from matplotlib import image
from matplotlib import pyplot
from pathlib import Path

class Dataset(object):
    '''
        A class for the dataset that will return data items as per the given index
    '''

    def __init__(self, annotation_file, transforms = None):
        '''
            Arguments:
            annotation_file: path to the annotation file
            transforms: list of transforms (class instances)
                        For instance, [<class 'RandomCrop'>, <class 'Rotate'>]
        '''
        self.annotation_file = annotation_file
        self.transforms = transforms
        self.data_list = []
        with open(self.annotation_file) as f:
            for line in f:
                img_data = json.loads(line)
                gt_bboxes = np.array([])
                
                for item in img_data['bboxes']:

                    # getting the list [category_id, x1, y1, x2, y2]
                    item_info = np.append(item['category_id'], item['bbox'])

                    if(len(gt_bboxes) == 0):
                        gt_bboxes = item_info
                        # after adding one element we reshape it to 1 x 5 2D array
                        gt_bboxes = gt_bboxes.reshape(1,5)
                    else:
                        # stacking the size 5 arrays to get the N x 5 gt_bboxes array
                        gt_bboxes = np.vstack((gt_bboxes,item_info))
                
                path_afile = Path(annotation_file)
                dir_data = str(path_afile.parent.absolute())
                img = image.imread(dir_data + '\\' + img_data['img_fn'])
                
                # applying the transforms sequentially on the current image
                if(self.transforms != None):
                    for transform in self.transforms:
                        img = transform(img)

                # scaling the values in the array to be within [0, 1]
                img = np.interp(img, (img.min(), img.max()), (0, +1))

                # converting image to (3, H, W) format
                img = np.transpose(img, (2,0,1))
                
                self.data_list.append([img, gt_bboxes])
        

    def __len__(self):
        '''
            return the number of data points in the dataset
        '''
        return len(self.data_list)
        

    def __getitem__(self, idx):
        '''
            return the dataset element for the index: "idx"
            Arguments:
                idx: index of the data element.

            Returns: A dictionary with:
                image: image (in the form of a numpy array) (shape: (3, H, W))
                gt_bboxes: N X 5 array where N is the number of bounding boxes, each 
                            consisting of [class, x1, y1, x2, y2]
                            x1 and x2 lie between 0 and width of the image,
                            y1 and y2 lie between 0 and height of the image.

            You need to do the following, 
            1. Extract the correct annotation using the idx provided.
            2. Read the image and convert it into a numpy array (wont be necessary
                with some libraries). The shape of the array would be (3, H, W).
            3. Scale the values in the array to be with [0, 1].
            4. Create a dictonary with both the image and annotations
            4. Perform the desired transformations.
            5. Return the transformed image and annotations as specified.
        '''
        img_dict_idx = {}
        img_dict_idx['image'] = self.data_list[idx][0]
        img_dict_idx['annotations'] = self.data_list[idx][1]
        return img_dict_idx

        


#Imports
import numpy as np
from skimage.transform import rotate

class RotateImage(object):
    '''
        Rotates the image about the centre of the image.
    '''

    def __init__(self, degrees):
        '''
            Arguments:
            degrees: rotation degree.
        '''
        self.degrees = degrees

    def __call__(self, sample):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        image_rotated = rotate(sample, self.degrees, resize = True)
        return image_rotated



from setuptools import setup, find_packages

VERSION = '1.1' 
DESCRIPTION = 'My package'
LONG_DESCRIPTION = 'My first Python package but with a longer description'

# Setting up
setup(
        name="my_package_19CS10073", 
        version=VERSION,
        author="Rajat Bachhawat",
        author_email="<myemail@abc.com>",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['matplotlib', 'opencv-python'],
        keywords=['python', 'first-package'],
        classifiers= [
            "Intended Audience :: Education",
            "Programming Language :: Python :: 3",
            "Operating System :: Microsoft :: Windows",
        ]
)



#Imports
import numpy as np

class FlipImage(object):
    '''
        Flips the image.
    '''

    def __init__(self, flip_type = 'horizontal'):
        '''
            Arguments:
            flip_type: 'horizontal' or 'vertical' Default: 'horizontal'
        '''
        self.flip_type = flip_type

        
    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        image_flipped = np.array([])
        if(self.flip_type == 'horizontal'):
            image_flipped = np.flip(image, axis = 0)
        elif(self.flip_type == 'vertical'):
            image_flipped = np.flip(image, axis = 1)
        
        return image_flipped


#Imports
import numpy as np
from scipy.ndimage.filters import gaussian_filter

class BlurImage(object):
    '''
        Applies Gaussian Blur on the image.
    '''

    def __init__(self, radius):
        '''
            Arguments:
            radius (int): radius to blur
        '''
        self.radius = radius
        

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL Image)

            Returns:
            image (numpy array or PIL Image)
        '''
        image_blurred = gaussian_filter(image, sigma=[self.radius, self.radius, 0])
        return image_blurred

